msg_version:	db	"v0.0.1"
msg_crlf:	db 	13, 10, 0
msg_print_sect:	db	"Sct=", 0

boot_dev:	db 0
sect_per_track	dw	18
sides		dw	2

current_sector	dw	1

NULL	equ	0x0000
END	equ	0x00
SCT_MIN	equ	1
SCT_MAX	equ	200


k0_start:
	jmp outer_interpreter
	cli		; interrupts disabled
			; set source
	mov ax, cs
	mov ds, ax	; ds = source segment(0)
	mov si, 0x7C00	; si = source offset(7C00)
			; set destination
	mov ax, 0x1000
	mov es, ax	; es = destination segment(1000)
	mov di, 0x0000	; di = destination offset(0xFD00)

	mov cx, 512	; move 512 bytes

	rep movsb	; relocate boot loader from 0000:7C00 to 1000:0000

	sti
	jmp 0x1000:new_location	; jump to new location

new_location:
	mov ax, cs
	mov ds, ax
	mov ss, ax
	mov es, ax
	cli
	mov sp, 0xF0F0
	push bp
	mov bp,  sp
	sti
	mov byte [boot_dev], dl


;-----------------------------------------------	
;			; interpreter

outer_interpreter:
	mov si, threaded_interpreter
	NEXT
threaded_interpreter:
interp_loop:
	dw outer_status
	dw get_char
	dw inner_find
	dw prim_skip_zero,  inner_execute
	dw prim_jmp, interp_loop


	
;-------------------------------------------------
			; called by outer interpreter

outer_prev_sct:
dumb_start:
	ENTER
dumb_end:
	dw change_sector, -1
	LEAVE

jmp_length	equ	dumb_end - dumb_start
	
outer_next_sct:
	ENTER
	dw change_sector, 1
	LEAVE

outer_prev_10:
	ENTER
	dw change_sector, -10
	LEAVE

outer_next_10:
	ENTER
	dw change_sector, 10
	LEAVE

outer_load:
	ENTER
	dw prim_load         
	dw prim_print, BUF_ADR
	dw prim_print, msg_crlf
	LEAVE

outer_run:
	ENTER
	dw prim_load
	dw prim_test_exec
	dw prim_skip_nzero, prim_run

	LEAVE

outer_status:
	ENTER
	dw prim_print, msg_version
	dw inner_print_sector
	LEAVE

; ------------------------------------------------
			; inner gears
inner_enter:
	push si
	add ax, jmp_length	; jmp next is 3 bytes long (AFAIK)
	mov si, ax
next:
	lodsw
	jmp ax


inner_leave:
	pop si
	jmp next


inner_execute:
	;mov di, bx
	;mov ax, [di]
	mov ax, bx
	jmp ax

inner_find:
	push si
	mov si, cmd_table
get_cmd:
	lodsb			
	cmp al, END		
	jz not_found
test_cmd:
	cmp al, bl		
	je found
	lodsw
	jmp get_cmd
not_found:
	mov bx, 0
	jmp end_find
found:
	lodsw
	mov bx,ax
end_find:
	pop si
	NEXT

;---------------------------------------------------------
			; inner words

prim_print:
	lodsw
	push si
	mov si, ax
next_one:	
	lodsb	; al := [ si ]
	cmp al, 0	; compare al with char(0)
	jz end_tty_out_asciiz

	call tty_out_char
	jmp next_one

end_tty_out_asciiz:
	pop si
	NEXT


prim_print_current:
	mov ax, word [current_sector]
	mov bx, 0xFFFF
	push BX
	MOV  BX, 10
CONV_decompose:
        xor dx, dx
        DIV  BX

        PUSH DX

	CMP  AX, 0
	JNE  CONV_decompose

conv_pop:
	pop AX
	cmp AX, 0xFFFF
	je fin_conv

	add AL, '0'
	call tty_out_char
	jmp conv_pop
fin_conv:

	NEXT

;---------------------------------------------------
	; print sector

inner_print_sector:	
	ENTER
	dw prim_print, msg_print_sect
	dw prim_print_current
	dw prim_print, msg_crlf
	LEAVE

;---------------------------------------------------
	; change sector
change_sector:
	lodsw
	add ax, word [current_sector]
	cmp ax, SCT_MAX+1
	jl not_too_high
	mov ax, SCT_MIN
not_too_high:
	cmp ax, SCT_MIN-1
	jg not_too_low
	mov ax, SCT_MAX
not_too_low:
	mov word [current_sector], ax
	NEXT

prim_load:
	call load_it
	;pop bx
	NEXT

prim_skip_zero:
	cmp bx, 0
	je do_skip
	NEXT

prim_skip_nzero:
	cmp bx, 0
	je next
do_skip:
	inc si
	inc si
	NEXT

prim_jz:
	cmp bx, 0
	jnz do_skip

prim_jmp:
	lodsw
	mov si, ax
	NEXT

%macro lba_to_chs 0
	mov bx, ax			; Save logical sector

	xor dx, dx			; First the sector
	div word [sect_per_track]
	add dl, 01h			; Physical sectors start at 1
	mov cl, dl			; Sectors belong in CL for int 13h
	mov ax, bx

	xor dx, dx			; Now calculate the head
	div word [sect_per_track]
	xor dx, dx
	div word [sides]
	mov dh, dl			; Head/side
	mov ch, al			; Track

	mov dl, byte [boot_dev]
        mov al, 1           ; Load 1 sector 


%endmacro 

	

prim_test_exec:
	mov bx, 0
	mov si, BUF_ADR
	lodsw	
	cmp AX, 0x9090
	jne next	;--------- exit condition
	lodsw
	cmp AX, 0x09E9
	jne next	;--------- exit condition
	mov BX, 1
	NEXT
	
prim_run:
	; call BUF_ADR
	NEXT

load_it:
        mov ax, ds       ; ES:BX = 1000:0800
        mov es, ax       ;

l2hts:			; Calculate head, track and sector settings for int 13h
			; IN: logical sector in AX, OUT: correct registers for int 13h
	mov ax, word [current_sector]
	lba_to_chs 
	mac_load nop
	ret


;------------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;

;-----------------------------------------------------------------------------
;

